import importlib
import json

import requests
from bs4 import BeautifulSoup


def parse(function):
    function._soupstar = True
    return function


class BeautifulSoupStar(BeautifulSoup):
    """
    An enhanced BeautifulSoup object, with additional helper methods
    """

    def find_text(self, text):
        regex = re.compile(text)
        result = self.find(text=re.compile(regex))
        if result:
            return result.parent

    def find_class(self, klass):
        regex = re.compile(klass)
        return self.find_all(attrs={'class': regex})

    def links(self, format=True):
        return [a.get('href') for a in self.find_all('a')]

    def title(self):
        return self.find('title').text


# TODO: methodology for catching parser errors
# TODO: methodology for using a "save" function
# TODO: figure out passing params to the init
#   Use the params passed by the caller
#   Otherwise use the attributes defined on the module
# TODO: figure out running parsers from anywhere, ie relative paths
class SoupStar(object):

    def __init__(self, module, url=None):
        self.module_string = module
        self.module = importlib.import_module(module)
        self.url = url or getattr(self.module, 'url')
        self.loaded = False
        self.errors = {}

    @property
    def parsers(self):
        # Any function from the module that has been tagged with @parse
        return {k: v for k, v in vars(self.module).items() \
                if hasattr(v, '_soupstar')}

    def load(self):
        self.response = requests.get(self.url)
        self.soup = BeautifulSoupStar(self.response.content,
                                  features="html.parser")
        self.soup.response = self.response
        self.soup.url = self.url
        self.loaded = True

    def to_tuples(self):
        if not self.loaded:
            self.load()

        for parser_name, parser_func in self.parsers.items():
            yield parser_name, parser_func(self.soup)

    def to_dict(self):
        return {
            'data': dict(self.to_tuples()),
            'status': self.response.status_code,
            'url': self.url,
            'errors': self.errors
        }

    def to_json(self):
        return json.dumps(self.to_dict(), default=str)



"""
Extract article links and article metadata from nytimes.com

"""

import re

from soupstars import parse


@parse
def title(self):
    """
    The title of the article.
    """

    return self.h1.text

@parse
def author(self):
    """
    The author(s) of the article.
    """

    return self.find(attrs={'itemprop': 'author creator'}).text



"""
Extract metadata from economist index and article pages

"""

import dateparser

from soupstars import parse


@parse
def date(soup):
    "The date of the article"

    string = self.find(attrs={'class': 'print-edition__main-title-header__date'})
    text = string.text.strip()
    return dateparser.parse(text).date()

@parse
def articles(soup):
    "The paths of the article urls"

    spans = self.find_all('span', attrs={'class': 'print-edition__link-title'})
    return [span.parent['href'] for span in spans]

@parse
def num_articles(soup):
    "The number of articles found on the page"

    return len(article_paths(soup))



import os
import requests


class Client(object):
    """
    Wrapper for the SoupStars web API.

    >>> from soupstars import Client
    >>> client = Client()
    >>> resp = client.status()
    >>> resp.json['healthy']
    True
    """


    # TODO: generic way of loading config vars
    def __init__(self, host=None, token=None):
        self.host = host or os.environ.get('SOUPSTARS_API_HOST') or 'https://soupstars-cloud.herokuapp.com'
        self.token = token or os.environ.get('SOUPSTARS_API_TOKEN')

    def send(self, path, method='POST', data=None, files=None):
        assert path[0] is '/', f"path must start with `/`, but received {path}"
        url = self.host + path
        headers = {
            'Authorization': self.token
            # TODO: add the version of soupstars, requests, python in use
        }
        resp = requests.request(url=url, method=method, json=data,
                                headers=headers, files=files)
        return resp

    def register(self, email, password):
        data = {
            'email': email,
            'password': password
        }
        return self.send('/register', data=data)

    def health(self):
        return self.send('/health', method='GET')

    def ls(self):
        return self.send('/list')

    def profile(self):
        return self.send('/profile')

    def push(self, module):
        file_data = open(module, 'rb').read()
        files = {module: file_data}
        return self.send('/push', files=files)

    def pull(self, module):
        data = {'name': module}
        return self.send('/pull', data=data)

    def run(self, module):
        data = {'name': module}
        return self.send('/run', data=data)



from setuptools import setup, find_packages
from os import path
from io import open

import soupstars

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='soupstars',
    version=soupstars.__version__,
    description='Declarative web parsers',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://soupstars.readthedocs.org',
    author='Tom Waterman',
    author_email='tjwaterman99@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='scraping parsing beautifulsoup beautiful soup',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=[
        'beautifulsoup4==4.8.0',
        'requests==2.22',
        'click==7.0'
    ],
    entry_points = {
      'console_scripts': [
          'soupstars = soupstars.cli:cli',
      ],
  }
)
